package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import com.smith9.geiger.client.beans.Stat

/**
 * Created: 7/06/11
 * @author david
 */
class BatchingGeigerClient implements BatchingClient {
    LoaderClient client
    List<Stat> stats = []
    Stat lastStatCommitted
    int batchSize

    BatchingGeigerClient(int batchSize, LoaderClient client) {
        this.client = client
        this.batchSize = batchSize
    }

    Stat getLastStatCommitted() {
        return lastStatCommitted
    }

    boolean batch(List<Stat> statsList) {
        stats.addAll(statsList)
        if (stats.size() >= batchSize) {
            send()
            return true
        }
        return false
    }

    boolean batchRemaining() {
        if (!stats.isEmpty()) {
            lastStatCommitted = client.send(stats)
            stats.clear()
            return true
        }
        return false
    }

    private void send() {
        int statCount = stats.size()
        int startIndex = 0
        int endIndex = 0
        while ((endIndex = startIndex + batchSize) <= statCount) {
            List<Stat> batch = stats.subList(startIndex, endIndex)
            lastStatCommitted = client.send(batch)
            startIndex = endIndex
        }
        //
        List<Stat> tail = new ArrayList(stats.subList(startIndex, statCount))
        stats.clear()
        stats = tail
    }

    int size() {
        return stats.size()
    }
}
