package com.smith9.geiger.client

import com.smith9.geiger.client.beans.Duration
import org.apache.commons.httpclient.methods.PostMethod
import org.apache.commons.httpclient.HttpClient

import org.joda.time.DateTime

import org.slf4j.Logger
import org.slf4j.LoggerFactory
import com.smith9.geiger.client.beans.Stat
import com.smith9.geiger.client.beans.Count
import com.smith9.geiger.client.beans.CountList
import simplygroovy.bean.Period

/**
 * Created: Mar 17, 2011
 * @author david
 */

class GeigerClient implements LoaderClient {
    private static final Logger LOG = LoggerFactory.getLogger(GeigerClient.class)
    private URL url
    private long totalSent
    private long totalDuration
//    private long batchesSent
    private HttpClient client

    public static void main(String[] args) {
        boolean loadCounts = false
        boolean loadDurations = false
        boolean calcStats = false
        GeigerClient client = new GeigerClient(new URL("http://localhost:8080/geiger"))
        BatchingGeigerClient batchingClient = new BatchingGeigerClient(100, client)

        if(calcStats) {
            String xml = client.executeQuery(new DateTime(2012, 4, 19, 22, 43, 0, 0), new DateTime(2012, 4, 19, 22, 46, 0, 0), "SOTL1", "EasyDoc", "stats", "by target type", "name", "group")
            println GeigerClient.getPercentDiffFromBaseLine(10, xml)
            Double percentileDiff95 = GeigerClient.getPercentDiffFromBaseLine(95, xml)
            println "95 percentile diff = $percentileDiff95"
        }
        if (loadDurations) {
            loadTestDurations(batchingClient)
        }
        if(loadCounts) {
            loadTestCounts()
        }
    }

    private static def loadTestCounts() {
        GeigerClient client = new GeigerClient(new URL("http://localhost:8080/geiger"))
        BatchingGeigerClient batchingClient = new BatchingGeigerClient(100, client)
        LinkedHashMap<String, Integer> limitsPerHour = [0: 1, 1: 1, 2: 1, 3: 2, 4: 2, 5: 3, 6: 3, 7: 14, 8: 22, 9: 44, 10: 88, 11: 92, 12: 66, 13: 55, 14: 82, 15: 99, 16: 62, 17: 34, 18: 22, 19: 10, 20: 4, 21: 3, 22: 2, 23: 1]
        CountList counts = new CountList()
        DateTime start = new DateTime()
        int successCount = 0
        (2006..2008).each { int year ->
            println "year $year"
            (1..12).step(2) { int month ->
                println "month $month"
                DateTime firstDayOfMonth = new DateTime(year, month, 1, 0, 0, 0, 0)
                int lastDayOfMonth = firstDayOfMonth.dayOfMonth().withMaximumValue().dayOfMonth
                (1..lastDayOfMonth).step(2) { int day ->
                    println "day $day"
                    (0..23).step(5) { int hour ->
                        int limitForHour = limitsPerHour[hour]
                        (0..59).step(20) { int minute ->
                            DateTime date = new DateTime(year, month, day, hour, minute, 0, 0)
                            (1..2).each { int product ->
                                String subProductName = "Product$product"
                                ['fax', 'email'].each { String targetType ->
                                    int count = (Math.random() * limitForHour) + 1
                                    boolean failure = ++successCount % 500 == 0
                                    if (failure) {
                                        int failCount = Math.random() * 5
                                        counts.add(new Count(category: "target type", subCategory: targetType, period: Period.MINUTE, periodStart: date,
                                            productName: "EasyDoc", subProduct: subProductName, count: failCount, environment: "PROD", min: failCount, max: failCount, sum: failCount, failure: true))

                                        if (counts.size() >= 100) {
                                            batchingClient.batch(counts.counts)
                                            batchingClient.batchRemaining()
                                            counts.clear()
                                        }
                                    }
                                    counts.add(new Count(category: "target type", subCategory: targetType, period: Period.MINUTE, periodStart: date,
                                        productName: "EasyDoc", subProduct: subProductName, count: count, environment: "PROD", min: count, max: count, sum: count, failure: false))

                                    if (counts.size() >= 100) {
                                        batchingClient.batch(counts.counts)
                                        batchingClient.batchRemaining()
                                        counts.clear()
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    private static def loadTestDurations(BatchingGeigerClient batchingClient) {
        ["SOTL1", "SOTL2"].each { String env ->
            ["SUNBank", "PI"].each { String subProduct ->
                ["End to End", "JMS Time", "Render Time"].each { String measure ->
                    (1..10).each { int duration ->
                        1.times {
                            ["email", "fax"].each { String targetType ->
                                batchingClient.batch([new Duration(environment: env, product: "EasyDoc", subProduct: subProduct, startTime: new DateTime().minusMinutes(duration),
                                    duration: duration * Math.random(), measure: measure, grouping: "1234567", reference: "123:1", failure: false, metadata: [targetType: targetType, numberOfRenderings: "4"])])
                            }
                        }
                    }
                }
            }

        }
        batchingClient.batchRemaining()
    }

    public GeigerClient(URL url) {
        this.url = url
        client = new HttpClient()
//        batchesSent = 0
    }

    static Double getPercentDiffFromBaseLine(int percentile, String xml) {
        def root = new XmlSlurper().parseText(xml)
        String percentileAttr = root.percentDiffs.percentiles.@"p_$percentile".toString()
        if(percentileAttr.size() > 0) {
            return percentileAttr.toDouble()
        }
        return null
    }

    String executeQuery(DateTime start, DateTime end, String environment, String productName, String queryGroup, String queryName, String saveName, String saveGroup) {
        String xml
        PostMethod post = new PostMethod(url.toString() + "/services")
        try {
            post.addParameter("fromDate", start.toString())
            post.addParameter("toDate", end.toString())
            post.addParameter("productName", productName)
            post.addParameter("environment", environment)
            post.addParameter("queryGroup", queryGroup)
            post.addParameter("queryName", queryName)
            post.addParameter("saveName", saveName)
            post.addParameter("saveGroup", saveGroup)
            int status = client.executeMethod(post)
            if (status != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                LOG.error("Error executing query to $url query to post was: $queryName")
                LOG.error("status code " + post.getStatusCode())
                String response = post.getResponseBodyAsString()
                println response
                LOG.error(response)
            } else {
                xml = post.getResponseBodyAsString()
                println xml
            }
        } finally {
            // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
            post.releaseConnection()
        }
        return xml
    }

    Stat send(List<Stat> stats) {
        long start = System.currentTimeMillis()
        LoaderConverter converter = createConverter(stats)
        uploadStats(converter, stats)
        logTimeTaken(System.currentTimeMillis() - start, stats.size())
        return stats.last()
    }

    void logTimeTaken(Long duration, int count) {
        if (LOG.debugEnabled) {
            totalSent += count
            totalDuration += duration
            LOG.debug("Sent $count, 1 took ${duration / count / 1000}s, $totalSent averaged ${totalDuration / totalSent / 1000}s")
        }
    }

    private void uploadStats(LoaderConverter converter, List<Stat> stats) {
        PostMethod post = new PostMethod(url.toString() + "/" + converter.loadMethod)
        try {
            String xml = converter.toXml(stats)
            post.addParameter("xml", xml)
            int status = client.executeMethod(post)
            if (status != org.apache.commons.httpclient.HttpStatus.SC_OK) {
                LOG.error("Error uploading stats to $url xml to post was: $xml")
                LOG.error("status code " + post.getStatusCode())
                LOG.error(post.getResponseBodyAsString())
            } else {
                // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
                cleanupResponseBody(post)
            }
        } finally {
            // cleanup - see http://hc.apache.org/httpclient-3.x/tutorial.html
            post.releaseConnection()
        }
    }

//    void cleanupClient() {
    //        if(batchesSent % 100 == 0) {
    //            LOG.debug("creating a new HttpClient to free memory/resources")
    //            client = new HttpClient()
    //        }
    //    }

    private void cleanupResponseBody(PostMethod post) {
        InputStream response = post.getResponseBodyAsStream()
        BufferedReader bufferedResponse = new BufferedReader(new InputStreamReader(response));
        String line;
        while ((line = bufferedResponse.readLine()) != null) {
            // TODO comment this out
            println line
        }
        bufferedResponse.close();
    }

    private LoaderConverter createConverter(List<Stat> stats) {
        if (stats.first() instanceof Duration) {
            return new DurationConverter()
        } else {
            return new CountConverter()
        }
    }
}