package com.smith9.geiger.client.beans

import org.joda.time.DateTime
import simplygroovy.util.DateUtil
import simplygroovy.bean.Period
import org.apache.commons.beanutils.BeanUtils

/**
 * Created: 22/03/12
 * @author david
 */
class CountList {
    boolean accumulateAdds = true
    AddMode mode = AddMode.MERGE

    private List<Count> counts = []
    public List<Count> getCounts() {
        return Collections.unmodifiableList(counts)
    }
    public void clear() {
        counts.clear()
    }
    public int size() {
        return counts.size()
    }
    public Count remove(Count count) {
        Count found = find(count)
        if(found) {
            counts.remove(found)
        }
        return found
    }
    public void add(Count count) {
        levelDate(count)
        if(mode == AddMode.MERGE) {
            if(accumulateAdds) {
                mergeAccumulate(count)
            } else {
                merge(count)
            }
        } else {
            replace(count)
        }
    }

    void levelDate(Count count) {
        count.periodStart = DateUtil.level(count.periodStart, count.period)
    }

    private void mergeAccumulate(Count count) {
        switch(count.period) {
            case Period.MINUTE:
                merge(createCopy(count, Period.MINUTE))
            case Period.HOUR:
                merge(createCopy(count, Period.HOUR))
            case Period.DAY:
                merge(createCopy(count, Period.DAY))
            case Period.MONTH:
                merge(createCopy(count, Period.MONTH))
            case Period.YEAR:
                merge(createCopy(count, Period.YEAR))
        }
    }

    Count createCopy(Count count) {
        return BeanUtils.cloneBean(count) as Count
    }
    Count createCopy(Count count, Period period) {
        Count shallowCopy = createCopy(count)
        shallowCopy.period = period
        levelDate(shallowCopy)
        return shallowCopy
    }

    private void merge(Count count) {
        Count existing = find(count)
        if (existing) {
            existing.merge(count)
        } else {
            counts.add(count)
        }
    }

    private Count replace(Count count) {
        Count removed = remove(count)
        counts.add(count)
        return removed
    }

    public Count find(Count count) {
        Count shallowCopy = createCopy(count)
        levelDate(shallowCopy)
        String key = shallowCopy.key()
        return counts.find { Count currentCount ->
            currentCount.key() == key
        }
    }
}
